#!/usr/bin/env python

#-----------------------------------------------------------------------
# recovery.py
# Author: Bob Dondero
#-----------------------------------------------------------------------

import os
import sys
import random
import psycopg2
import dotenv

dotenv.load_dotenv()
DATABASE_URL = os.environ['DATABASE_URL']

#-----------------------------------------------------------------------

def main():

    if len(sys.argv) != 1:
        print('Usage: python ' + sys.argv[0], file=sys.stderr)
        sys.exit(1)

    try:
        with psycopg2.connect(DATABASE_URL) as connection:

            with connection.cursor() as cursor:

                for i in range(20):

                    cursor.execute('BEGIN')

                    stmt_str = "UPDATE orders "
                    stmt_str += "SET quantity = quantity+1 "
                    stmt_str += "WHERE isbn = '123' "
                    stmt_str += "AND custid = '222'"
                    cursor.execute(stmt_str)

                    # Simulate a HW/SW failure occurring randomly,
                    # on average every 5th time through the loop.
                    if random.randrange(5) == 0:
                        print('Simulated failure with i = %d' % i)
                        cursor.execute('ROLLBACK')
                        print('Transaction rolled back.')
                        continue

                    stmt_str = "UPDATE books "
                    stmt_str += "SET quantity = quantity-1 "
                    stmt_str += "WHERE isbn = '123'"
                    cursor.execute(stmt_str)

                    cursor.execute('COMMIT')
                    print('Transaction %d committed.' % i)

    except Exception as ex:
        print(ex, file=sys.stderr)
        sys.exit(1)

#-----------------------------------------------------------------------

if __name__ == '__main__':
    main()
